//! 🚧 Experimental module 🚧
//!
//! This module contains unstable functions.
//!
//! Please note that these functions may be removed or changed disruptively without any announcement.
//!
//! If you are eager to use this module, please enable `experimental` feature flag.

mod parse_with_chimeiruiju;
mod parse_with_geolonia;
pub mod parser;
